package com.tomtom.business.ubi.sample;

import java.util.Date;

import android.app.Activity;
import android.bluetooth.BluetoothAdapter;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.os.Bundle;
import android.os.IBinder;
import android.util.Log;

import com.tomtom.business.commons.service.InterfaceServiceBinder;
import com.tomtom.business.ubi.commons.api.Core;
import com.tomtom.business.ubi.commons.api.CoreBootParameter;
import com.tomtom.business.ubi.commons.api.DisconnectReasonParameter;
import com.tomtom.business.ubi.commons.api.ServiceClient;
import com.tomtom.business.ubi.commons.api.ServiceClientCallback;
import com.tomtom.business.ubi.core.DirectCoreService;


/**
 * A simple sample Android Activity demonstrating the TomTom UBI SDK for Android. 
 * Please see the documentation in the library project for more information.
 *
 * Copyright (c) 2016 TomTom Telematics, all rights reserved.
 */
public class SampleActivity extends Activity implements ServiceClientCallback
{
    private static final String LOG_TAG = SampleActivity.class.getName();

    /**
     * The main entry point the UBI SDK.
     */
    private Core core;

    /**
     * Use this to initiate a certain function.
     */
    private ServiceClient serviceClient;

    /**
     * Keep a ref for unbinding...
     */
    private ServiceConnection serviceConnection;

    /**
     * Simple implementation of an Android service connection.
     */
    private final class MyServiceConnection implements ServiceConnection
    {
        @Override
        public void onServiceDisconnected(ComponentName name)
        {
        }

        @Override
        public void onServiceConnected(ComponentName name, IBinder service)
        {
            //Note the necessary cast! 
            @SuppressWarnings("unchecked")
            Core core = ((InterfaceServiceBinder<Core>) service).getService();
            init(core);
        }
    }

    @Override
    protected void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);
        this.setContentView(R.layout.sample_activity);

        Log.i(LOG_TAG, "Binding to service...");
        this.serviceConnection = new MyServiceConnection();

        Intent bindIntent = new Intent(this, DirectCoreService.class);
        this.bindService(bindIntent, this.serviceConnection, Context.BIND_AUTO_CREATE);
    }

    /**
     * Called after the Android service hosting the Core has been started and the client successfully bound to it.
     */
    private void init(Core core)
    {
        //keep the core to query the states of the initialization of the service and the connection to dongle and server 
        this.core = core;

        //register a callback to be informed about events and other things that need to be handled
        this.serviceClient = this.core.getServiceClient();
        this.core.addCallback(this);

        //start the core
        this.bootCore();
    }

    /**
     * Starts the Core.
     */
    private void bootCore()
    {
        //request proper credentials from TTB (user, pass)
        //replace the Bluetooth address with the address of your dongle
        //optional: replace the configured end-points with alternative end-points
        CoreBootParameter coreBootParameter = CoreBootParameter.builder()
            .setUserName("<REPLACE_WITH_VALID_USERNAME>")
            .setPassword("<REPLACE_WITH_VALID_PASSWORD>")
            .setApiKey("UBI-ROBIN-SINGLEOBJECT")
            .enableReset() // using this method deletes all queued messages and a re-activation is carried out, even if the dongle has already been activated, otherwise the activation and queue state is re-used between runs
            .setDongleBtDevice(BluetoothAdapter.getDefaultAdapter().getRemoteDevice("<REPLACE_WITH_VALID_DONGLE_BT_ADDRESS>"))
            .setWebfleetMessagingServerHostName("its-robin-gprs.business.tomtom.com")
            .setWebfleetMessagingServerPort(443)
            .setWebfleetConnectServerHostName("https://its-wfc.business.tomtom.com")
            .build();
        this.serviceClient.bootCore(coreBootParameter);

        /* see onActivationFinished or onError*/
    }

    /**
     * Unbind from the Core service to release any resources.
     */
    @Override
    protected void onDestroy()
    {
        Log.i(LOG_TAG, "Unbinding service...");
        this.unbindService(this.serviceConnection);
        super.onDestroy();
    }

    @Override
    public void onActivationFinished()
    {
        Log.i(LOG_TAG, "Activation finished!");

        //now call the desired function in the service client
        this.serviceClient.startSync(); /* onDownloadStatusUpdate() or onClockSyncReasonForDownloadRequired() is called, if the data was successfully uploaded to the server onUploadToServerFinished() is called */

        //Examples:
        //this.serviceClient.changeOdometer(new OdometerValueParameter(50000 /*meter*/)); /* onOdometerChange() is called */
        //this.serviceClient.requestCoreState(); /* onCoreState() is called */ 
        //this.serviceClient.requestDeviceStatus(); /* onDeviceStatus() is called */
        //this.serviceClient.requestObdDtcList(); /* onObdDtcList() is called */
        //this.serviceClient.startObdRealtimeTransmission(); /* onObdRealTimeDatagram() is called */
        //this.serviceClient.stopObdRealtimeTransmission(); /* OBD real-time transmission is stopped */

        // send FCM token to WEBFLEET
        /*
        String fcmToken = FirebaseInstanceId.getInstance().getToken();
        // Please use a FirebaseInstanceIdService instead!
        // It's done this way to keep this example simple.
        if (fcmToken != null)
        {
            String senderId = getString(R.string.gcm_defaultSenderId);
            this.serviceClient.sendFcmRegistrationToken(new FcmRegistrationParameter(senderId, fcmToken));
        }
        */
    }

    @Override
    public void onActivationStart()
    {
        Log.i(LOG_TAG, "Starting activation...");
    }

    @Override
    public void onClockSyncReasonForDownloadRequired()
    {
        Log.i(
            LOG_TAG,
            "A reason for the synchronisation of the clock in the dongle was requested, sending reason with id '1'...");
        this.serviceClient.sendClockSyncReasonForDownload(new DisconnectReasonParameter(1));
    }

    @Override
    public void onClockSyncSuccesful(ClockSyncForActionResponse response)
    {
        Log.i(LOG_TAG, "Clock sync for action '" + response.clockSyncReasonForAction + "' was succesful.");
    }

    @Override
    public void onConnectionStatusUpdate(ConnectionStatusResponse response)
    {
        Log.i(LOG_TAG, "Connection state for '" + response.endPoint + "' has changed to '" + response.connectionStatus + "'.");
    }

    @Override
    public void onCoreBooted()
    {
        Log.i(LOG_TAG, "UBI Core has been booted...");
    }

    @Override
    public void onCoreState(CoreStateResponse response)
    {
        Log.i(LOG_TAG, "The internal state of the core was returned.");
    }

    @Override
    public void onCrashDetectedDatagram(CrashDetectedDatagramResponse response)
    {
        Log.i(LOG_TAG, "A crash was detected!");
    }

    @Override
    public void onDeviceStatus(DeviceStatusResponse response)
    {
        Log.i(LOG_TAG, "The device status was returned.");
    }

    @Override
    public void onDownloadStatusUpdate(DownloadStatusResponse response)
    {
        Log.i(LOG_TAG, "Status of download of UBI data has changed to: '" + response.downloadProgressState + "'");
    }

    @Override
    public void onError(UbiErrorCodeResponse response)
    {
        Log.e(LOG_TAG, "An error occured: [" + response.errorCode.errorCode + "] - '" + response.errorCodeDescription + "'");
    }

    @Override
    public void onFirmwareUpdateAvailable(FirmwareUpdateAvailableResponse response)
    {
        Log.i(LOG_TAG, "A firmware update is available.");
    }

    @Override
    public void onFirmwareUpdateStatusUpdate(FirmwareUpdateStatusResponse response)
    {
        Log.i(LOG_TAG, "Status of firmware update has changed to: '" + response.firmwareUpdateStatus + "'");
    }

    @Override
    public void onObdDtcList(ObdDtcListResponse response)
    {
        Log.i(LOG_TAG, "A list of DTC's has been returned.");
    }

    @Override
    public void onObdRealTimeDatagram(ObdRealTimeDatagramResponse response)
    {
        Log.i(LOG_TAG, "A OBD real-time datagram has been returned.");
    }

    @Override
    public void onOdometerChange(OdometerChangeResponse response)
    {
        Log.i(LOG_TAG, "Changing the odometer in the dongle returned with: '" + response.odometerChangeStatus + "'");
    }

    @Override
    public void onOutboundQueueStatusUpdate(OutboundQueueStatusUpdateResponse response)
    {
        Log.i(LOG_TAG, "The size of the outbound queue '" + response.outboundQueue + "' is " + response.queueSize + ".");
    }

    @Override
    public void onTransmission(TransmissionResponse response)
    {
        Log.i(LOG_TAG, "A message was transmitted: '" + response.transmissionState + "'.");
    }

    @Override
    public void onUploadToServerFinished(UploadToServerFinishedResponse response)
    {
        Log.i(
            LOG_TAG,
            "The upload of UBI data from the dongle to the server was finally successful. It contains all data till "
                + new Date(response.lastUbiDataTime) + ".");
    }

    @Override
    public void onDrivingProfileRealTimeDatagram(DrivingProfileRealTimeDatagramResponse response)
    {
        Log.i(LOG_TAG, "Driving provile realtime datagram received.'");
    }

    @Override
    public void onEngineStatusUpdate(EngineStatusResponse response)
    {
        Log.i(LOG_TAG, "Engine status updated to: '" + response.isEngineOn + "'");
    }
}