package com.tomtom.business.ubi.sample;

import android.app.Service;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.BroadcastReceiver;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.ServiceConnection;
import android.os.Binder;
import android.os.IBinder;
import android.util.Log;

import com.tomtom.business.commons.service.InterfaceServiceBinder;
import com.tomtom.business.ubi.commons.api.ConnectionStatus;
import com.tomtom.business.ubi.commons.api.Core;
import com.tomtom.business.ubi.commons.api.CoreBootParameter;
import com.tomtom.business.ubi.commons.api.ServiceClient;
import com.tomtom.business.ubi.commons.api.ServiceClientCallback;
import com.tomtom.business.ubi.core.DirectCoreService;

/**
 * A simple sample Android service demonstrating the TomTom UBI SDK for Android. 
 * Please see the documentation in the library project for more information.
 *
 * Ensure having specified the credentials and Bluetooth address of the dongle properly! 
 *
 * Copyright (c) 2016 TomTom Telematics, all rights reserved.
 */
public class SampleService extends Service implements ServiceClientCallback
{
    
    private static final String SAMPLE_DONGLE_BT_ADDRESS = "<REPLACE_WITH_VALID_DONGLE_BT_ADDRESS>";

    public class SampleServiceBinder extends Binder
    {
        SampleService getService() 
        {
            return SampleService.this;   
        }
    }

    private SampleServiceBinder binder = new SampleServiceBinder();
    
    private static final String LOG_TAG = SampleActivity.class.getName();
    
    /**
     * The main entry point the UBI SDK.
     */
    private Core core;
    
    /**
     * Use this to initiate a certain function.
     */
    private ServiceClient serviceClient;

    /**
     * Keep a ref for unbinding...
     */
    private ServiceConnection serviceConnection;

    private BroadcastReceiver btReceiver;
    
    /**
     * Simple implementation of an Android service connection.
     */
    private final class MyServiceConnection implements ServiceConnection
    {
        @Override
        public void onServiceDisconnected(ComponentName name)
        {
        }

        @SuppressWarnings("unchecked")
        @Override
        public void onServiceConnected(ComponentName name, IBinder service)
        {
            Log.i(LOG_TAG, "UBI service bound...");
            SampleService.this.core = ((InterfaceServiceBinder<Core>) service).getService();
            init();
        }
    }    

    @Override
    public void onCreate()
    {
        super.onCreate();
        
        //listen for incoming BT events (ACTION_ACL_CONNECTED)
        this.registerReceiver(this.btReceiver = new BroadcastReceiver()
        {
            @Override
            public void onReceive(Context context, Intent intent)
            {
                String btAddressViaAclConnectedEvent = ((BluetoothDevice)intent.getParcelableExtra(BluetoothDevice.EXTRA_DEVICE)).getAddress();
                
                //initiate a reconnection using the service client if the dongle is in range
                if(btAddressViaAclConnectedEvent.equals(SAMPLE_DONGLE_BT_ADDRESS))
                {
                    Log.i(LOG_TAG, "Got connection attempt from dongle, BT = '" + btAddressViaAclConnectedEvent + "'...");
                    
                    if(serviceClient != null)
                    {
                        serviceClient.reconnectToDongle();
                    }
                }
                else
                {
                    Log.i(LOG_TAG, "Got a connection attempt from another BT device with  '" + btAddressViaAclConnectedEvent + "', ignoring...");
                }
            }
        }, new IntentFilter(BluetoothDevice.ACTION_ACL_CONNECTED));
    }
    
    /**
     * Called after the Android service hosting the Core has been started and the client successfully bound to it.
     */
    public void startCoreService()
    {
        Log.i(LOG_TAG, "Binding to UBI service...");
        this.serviceConnection = new MyServiceConnection();
        
        Intent bindIntent = new Intent(this, DirectCoreService.class);
        this.bindService(bindIntent, this.serviceConnection, Context.BIND_AUTO_CREATE);
    }    

    @Override
    public IBinder onBind(Intent intent)
    {
        return this.binder;
    }

    /**
     * Unbind from the Core service to release any resources.
     */
    @Override
    public void onDestroy()
    {
        Log.i(LOG_TAG, "Unbinding service...");
        this.unregisterReceiver(this.btReceiver);
        this.serviceClient.stopObdRealtimeTransmission();
        this.unbindService(this.serviceConnection);
        super.onDestroy();
    }       
    
    /**
     * Creates a local service client and boots the Core.
     */
    private void init()
    {
        Log.i(LOG_TAG, "Booting Core in UBI service...");
        //register a callback to be informed about events and other things that need to be handled
        this.serviceClient = this.core.getServiceClient();
        this.core.addCallback(this);
        
        //start the core
        this.bootCore();
    }
    
    /**
     * Starts the Core.
     */
    private void bootCore()
    {
        //request proper credentials from TTB (user, pass)
        //replace the Bluetooth address with the address of your dongle
        //optional: replace the configured end-points with alternative end-points
        CoreBootParameter coreBootParameter = CoreBootParameter.builder()
            .setUserName("<REPLACE_WITH_VALID_USERNAME>")
            .setPassword("<REPLACE_WITH_VALID_PASSWORD>")
            .setApiKey("UBI-ROBIN-SINGLEOBJECT")
            .enableReset() // true = all queued messages will be deleted and a re-activation is carried out, even if the dongle has already been activated, false = the activation and queue state is re-used between runs
            .setDongleBtDevice(BluetoothAdapter.getDefaultAdapter().getRemoteDevice(SAMPLE_DONGLE_BT_ADDRESS))
            .setWebfleetMessagingServerHostName("its-robin-gprs.business.tomtom.com")
            .setWebfleetMessagingServerPort(443)
            .setWebfleetConnectServerHostName("https://its-wfc.business.tomtom.com")
            .build();
        this.serviceClient.bootCore(coreBootParameter);
        
        /* see onActivationFinished or onError*/
    }    
    
    @Override
    public void onActivationFinished()
    {
        Log.i(LOG_TAG, "Activiation was successfully finished.");

        //For example, start the transmission of OBD real time data, after the activation was finished successfully 
        this.serviceClient.startObdRealtimeTransmission();
    }

    @Override
    public void onActivationStart()
    {
        Log.i(LOG_TAG, "Starting activiation...");
    }

    @Override
    public void onClockSyncReasonForDownloadRequired()
    {
    }

    @Override
    public void onClockSyncSuccesful(ClockSyncForActionResponse clockSyncReasonForActionResponse)
    {
    }

    @Override
    public void onConnectionStatusUpdate(ConnectionStatusResponse connectionStatusResponse)
    {
        Log.i(LOG_TAG, "Connection state of '" + connectionStatusResponse.endPoint + "' changed to '" + connectionStatusResponse.connectionStatus + "'");
        
        if (connectionStatusResponse.endPoint == EndPoint.DONGLE && connectionStatusResponse.connectionStatus == ConnectionStatus.CONNECTED)
        {
            //Restart the transmission of OBD real time data after being reconnected to the dongle  
            this.serviceClient.startObdRealtimeTransmission();
        }
    }

    @Override
    public void onCoreBooted()
    {
    }

    @Override
    public void onCoreState(CoreStateResponse coreStateResponse)
    {
    }

    @Override
    public void onCrashDetectedDatagram(CrashDetectedDatagramResponse crashDetectedDatagramResponse)
    {
    }

    @Override
    public void onDeviceStatus(DeviceStatusResponse deviceStatusResponse)
    {
    }

    @Override
    public void onDownloadStatusUpdate(DownloadStatusResponse downloadStatusResponse)
    {
    }

    @Override
    public void onError(UbiErrorCodeResponse ubiErrorCodeResponse)
    {
    }

    @Override
    public void onFirmwareUpdateAvailable(FirmwareUpdateAvailableResponse firmwareUpdateAvailableResponse)
    {
    }

    @Override
    public void onFirmwareUpdateStatusUpdate(FirmwareUpdateStatusResponse firmwareUpdateStatusResponse)
    {
    }

    @Override
    public void onObdDtcList(ObdDtcListResponse obdDtcListResponse)
    {
    }

    @Override
    public void onObdRealTimeDatagram(ObdRealTimeDatagramResponse obdRealTimeDatagramResponse)
    {
    }

    @Override
    public void onOdometerChange(OdometerChangeResponse odometerChangeResponse)
    {
    }

    @Override
    public void onOutboundQueueStatusUpdate(OutboundQueueStatusUpdateResponse outboundQueueStatusUpdateResponse)
    {
    }

    @Override
    public void onTransmission(TransmissionResponse transmissionResponse)
    {
    }

    @Override
    public void onUploadToServerFinished(UploadToServerFinishedResponse uploadToServerFinishedResponse)
    {
    }

	@Override
	public void onDrivingProfileRealTimeDatagram(DrivingProfileRealTimeDatagramResponse drivingProfileRealTimeDatagramResponse) {
	}

	@Override
	public void onEngineStatusUpdate(EngineStatusResponse reponse) {
	}
}
